@@ -22,6 +22,11 @@
 #include <unistd.h>
 #include <string.h>
 
+/* For the thumbnail size tests in test_download_thumbnail() */
+#ifdef HAVE_GDK_PIXBUF
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#endif
+
 #include "gdata.h"
 #include "common.h"
 
@@ -211,17 +216,44 @@ typedef struct {
 } TempDocumentData;
 
 static GDataDocumentsDocument *
-_set_up_temp_document (GDataDocumentsEntry *entry, GDataService *service)
+_set_up_temp_document (GDataDocumentsEntry *entry, GDataService *service, GFile *document_file)
 {
 	GDataDocumentsDocument *document, *new_document;
-	gchar *upload_uri;
+	GFileInfo *file_info;
+	GFileInputStream *file_stream;
+	GDataUploadStream *upload_stream;
+	GError *error = NULL;
 
-	/* Insert the document */
-	upload_uri = gdata_documents_service_get_upload_uri (NULL);
-	document = GDATA_DOCUMENTS_DOCUMENT (gdata_service_insert_entry (service, gdata_documents_service_get_primary_authorization_domain (),
-	                                                                 upload_uri, GDATA_ENTRY (entry), NULL, NULL));
-	g_assert (GDATA_IS_DOCUMENTS_DOCUMENT (document));
-	g_free (upload_uri);
+	/* Query for information on the file. */
+	file_info = g_file_query_info (document_file,
+	                               G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+	                               G_FILE_ATTRIBUTE_STANDARD_SIZE, G_FILE_QUERY_INFO_NONE, NULL, &error);
+	g_assert_no_error (error);
+
+	/* Prepare the upload stream */
+	upload_stream = gdata_documents_service_upload_document_resumable (GDATA_DOCUMENTS_SERVICE (service), GDATA_DOCUMENTS_DOCUMENT (entry),
+	                                                                   g_file_info_get_display_name (file_info),
+	                                                                   g_file_info_get_content_type (file_info), g_file_info_get_size (file_info),
+	                                                                   NULL, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_UPLOAD_STREAM (upload_stream));
+
+	g_object_unref (file_info);
+
+	/* Open the file */
+	file_stream = g_file_read (document_file, NULL, &error);
+	g_assert_no_error (error);
+
+	/* Upload the document */
+	g_output_stream_splice (G_OUTPUT_STREAM (upload_stream), G_INPUT_STREAM (file_stream),
+	                        G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, &error);
+	g_assert_no_error (error);
+
+	/* Finish the upload */
+	document = gdata_documents_service_finish_upload (GDATA_DOCUMENTS_SERVICE (service), upload_stream, &error);
+	g_assert_no_error (error);
+
+	g_object_unref (upload_stream);
 
 	/* HACK: Query for the new document, as Google's servers appear to modify it behind our back when creating the document:
 	 * http://code.google.com/a/google.com/p/apps-api-issues/issues/detail?id=2337. We have to wait a few seconds before trying this to allow the
@@ -233,6 +265,8 @@ _set_up_temp_document (GDataDocumentsEntry *entry, GDataService *service)
 	                                                                           G_OBJECT_TYPE (document), NULL, NULL));
 	g_assert (GDATA_IS_DOCUMENTS_DOCUMENT (new_document));
 
+	g_object_unref (document);
+
 	return new_document;
 }
 
@@ -240,13 +274,20 @@ static void
 set_up_temp_document_spreadsheet (TempDocumentData *data, gconstpointer service)
 {
 	GDataDocumentsSpreadsheet *document;
+	gchar *document_file_path;
+	GFile *document_file;
 
 	/* Create a document */
 	document = gdata_documents_spreadsheet_new (NULL);
 	gdata_entry_set_title (GDATA_ENTRY (document), "Temporary Document (Spreadsheet)");
 
-	data->document = _set_up_temp_document (GDATA_DOCUMENTS_ENTRY (document), GDATA_SERVICE (service));
+	document_file_path = g_strconcat (TEST_FILE_DIR, "test.ods", NULL);
+	document_file = g_file_new_for_path (document_file_path);
+	g_free (document_file_path);
 
+	data->document = _set_up_temp_document (GDATA_DOCUMENTS_ENTRY (document), GDATA_SERVICE (service), document_file);
+
+	g_object_unref (document_file);
 	g_object_unref (document);
 }
 
@@ -820,7 +861,8 @@ set_up_update_document (UpdateDocumentData *data, gconstpointer _test_params)
 {
 	const UpdateDocumentTestParams *test_params = _test_params;
 	GDataDocumentsText *document;
-	gchar *title;
+	gchar *title, *document_file_path;
+	GFile *document_file;
 
 	/* Create a document */
 	document = gdata_documents_text_new (NULL);
@@ -828,8 +870,13 @@ set_up_update_document (UpdateDocumentData *data, gconstpointer _test_params)
 	gdata_entry_set_title (GDATA_ENTRY (document), title);
 	g_free (title);
 
-	data->document = _set_up_temp_document (GDATA_DOCUMENTS_ENTRY (document), GDATA_SERVICE (test_params->service));
+	document_file_path = g_strconcat (TEST_FILE_DIR, "test.odt", NULL);
+	document_file = g_file_new_for_path (document_file_path);
+	g_free (document_file_path);
 
+	data->document = _set_up_temp_document (GDATA_DOCUMENTS_ENTRY (document), GDATA_SERVICE (test_params->service), document_file);
+
+	g_object_unref (document_file);
 	g_object_unref (document);
 }
 
@@ -960,6 +1007,46 @@ test_update (UpdateDocumentData *data, gconstpointer _test_params)
 	g_object_unref (updated_document);
 }
 
+typedef struct {
+	TempDocumentData parent;
+	GDataDocumentsDocument *new_document;
+} TempCopyDocumentData;
+
+static void
+set_up_copy_document (TempCopyDocumentData *data, gconstpointer service)
+{
+	/* Create a temporary document. */
+	set_up_temp_document_spreadsheet ((TempDocumentData*) data, service);
+
+	data->new_document = NULL;
+}
+
+static void
+tear_down_copy_document (TempCopyDocumentData *data, gconstpointer service)
+{
+	/* Delete the copied document */
+	delete_entry (GDATA_DOCUMENTS_ENTRY (data->new_document), GDATA_SERVICE (service));
+	g_object_unref (data->new_document);
+
+	/* Delete the folder */
+	tear_down_temp_document ((TempDocumentData*) data, service);
+}
+
+static void
+test_copy_document (TempCopyDocumentData *data, gconstpointer service)
+{
+	GError *error = NULL;
+
+	/* Copy the document */
+	data->new_document = gdata_documents_service_copy_document (GDATA_DOCUMENTS_SERVICE (service), data->parent.document, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (GDATA_IS_DOCUMENTS_SPREADSHEET (data->new_document));
+
+	/* Check their IDs are different but that their other properties (e.g. title) are the same. */
+	g_assert_cmpstr (gdata_entry_get_id (GDATA_ENTRY (data->parent.document)), !=, gdata_entry_get_id (GDATA_ENTRY (data->new_document)));
+	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (data->parent.document)), ==, gdata_entry_get_title (GDATA_ENTRY (data->new_document)));
+}
+
 typedef struct {
 	GDataDocumentsFolder *folder;
 	GDataDocumentsDocument *document;
@@ -1242,6 +1329,80 @@ test_download_document (TempDocumentsData *data, gconstpointer service)
 	_test_download_document (GDATA_DOCUMENTS_DOCUMENT (data->arbitrary_document), GDATA_SERVICE (service));
 }
 
+static void
+test_download_thumbnail (TempDocumentData *data, gconstpointer service)
+{
+	const gchar *thumbnail_uri;
+	GDataDownloadStream *download_stream;
+	gchar *destination_file_name, *destination_file_path;
+	GFile *destination_file;
+	GFileOutputStream *file_stream;
+	gssize transfer_size;
+	GError *error = NULL;
+
+	thumbnail_uri = gdata_documents_document_get_thumbnail_uri (data->document);
+
+	/* Google takes many minutes to generate thumbnails for uploaded documents, so with our current testing strategy of creating fresh documents
+	 * for each test, this particular test is fairly useless. */
+	if (thumbnail_uri == NULL) {
+		g_test_message ("Skipping thumbnail download test because document ‘%s’ doesn’t have a thumbnail.",
+		                gdata_documents_entry_get_resource_id (GDATA_DOCUMENTS_ENTRY (data->document)));
+		return;
+	}
+
+	/* Download the thumbnail to a file for testing (in case we weren't compiled with GdkPixbuf support) */
+	download_stream = GDATA_DOWNLOAD_STREAM (gdata_download_stream_new (GDATA_SERVICE (service), NULL, thumbnail_uri, NULL));
+	g_assert (GDATA_IS_DOWNLOAD_STREAM (download_stream));
+
+	/* Prepare a file to write the data to */
+	destination_file_name = g_strdup_printf ("%s_thumbnail.jpg", gdata_documents_entry_get_resource_id (GDATA_DOCUMENTS_ENTRY (data->document)));
+	destination_file_path = g_build_filename (g_get_tmp_dir (), destination_file_name, NULL);
+	g_free (destination_file_name);
+	destination_file = g_file_new_for_path (destination_file_path);
+	g_free (destination_file_path);
+
+	/* Download the file */
+	file_stream = g_file_replace (destination_file, NULL, FALSE, G_FILE_CREATE_REPLACE_DESTINATION, NULL, &error);
+	g_assert_no_error (error);
+	g_assert (G_IS_FILE_OUTPUT_STREAM (file_stream));
+
+	transfer_size = g_output_stream_splice (G_OUTPUT_STREAM (file_stream), G_INPUT_STREAM (download_stream),
+	                                        G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, NULL, &error);
+	g_assert_no_error (error);
+	g_assert_cmpint (transfer_size, >, 0);
+
+	g_object_unref (file_stream);
+	g_object_unref (download_stream);
+
+	/* Delete the file (shouldn't cause the test to fail if this fails) */
+	g_file_delete (destination_file, NULL, NULL);
+	g_object_unref (destination_file);
+
+#ifdef HAVE_GDK_PIXBUF
+	/* Test downloading all thumbnails directly into GdkPixbufs, and check that they're all the correct size */
+	{
+		GdkPixbuf *pixbuf;
+
+		/* Prepare a new download stream */
+		download_stream = GDATA_DOWNLOAD_STREAM (gdata_download_stream_new (GDATA_SERVICE (service), NULL, thumbnail_uri, NULL));
+		g_assert (GDATA_IS_DOWNLOAD_STREAM (download_stream));
+
+		/* Download into a new GdkPixbuf */
+		pixbuf = gdk_pixbuf_new_from_stream (G_INPUT_STREAM (download_stream), NULL, &error);
+		g_assert_no_error (error);
+		g_assert (GDK_IS_PIXBUF (pixbuf));
+
+		g_object_unref (download_stream);
+
+		/* Check the dimensions are as expected. */
+		g_assert_cmpint (gdk_pixbuf_get_width (pixbuf), ==, 10);
+		g_assert_cmpint (gdk_pixbuf_get_height (pixbuf), ==, 10);
+
+		g_object_unref (pixbuf);
+	}
+#endif /* HAVE_GDK_PIXBUF */
+}
+
 static void
 test_access_rule_insert (TempDocumentData *data, gconstpointer service)
 {
@@ -1699,6 +1860,8 @@ main (int argc, char *argv[])
 
 		g_test_add ("/documents/download/document", TempDocumentsData, service, set_up_temp_documents, test_download_document,
 		            tear_down_temp_documents);
+		g_test_add ("/documents/download/thumbnail", TempDocumentData, service, set_up_temp_document_spreadsheet, test_download_thumbnail,
+		            tear_down_temp_document);
 
 		/* Test all possible combinations of conditions for resumable updates. */
 		{
@@ -1746,6 +1909,12 @@ main (int argc, char *argv[])
 		g_test_add ("/documents/query/all_documents/async/cancellation", GDataAsyncTestData, service, set_up_temp_documents_async,
 		            test_query_all_documents_async_cancellation, tear_down_temp_documents_async);
 
+		g_test_add ("/documents/copy", TempCopyDocumentData, service, set_up_copy_document, test_copy_document, tear_down_copy_document);
+		/*g_test_add ("/documents/copy/async", GDataAsyncTestData, service, set_up_folders_add_to_folder_async,
+		            test_folders_add_to_folder_async, tear_down_folders_add_to_folder_async);
+		g_test_add ("/documents/copy/async/cancellation", GDataAsyncTestData, service, set_up_folders_add_to_folder_async,
+		            test_folders_add_to_folder_async_cancellation, tear_down_folders_add_to_folder_async);*/
+
 		g_test_add ("/documents/folders/add_to_folder", FoldersData, service, set_up_folders_add_to_folder,
 		            test_folders_add_to_folder, tear_down_folders_add_to_folder);
 		g_test_add ("/documents/folders/add_to_folder/async", GDataAsyncTestData, service, set_up_folders_add_to_folder_async,
